use std::{env, time::Duration};

use rs48_lib::{game::Game, grid_displayer::GridDisplayer, prelude::*};
use termion::{clear, cursor};
use tokio::{
    io::AsyncWriteExt,
    net::{tcp::OwnedWriteHalf, TcpListener},
    time::sleep,
};

#[derive(Debug, Clone)]
struct Config {
    delay: Duration,
}

async fn game(mut write: OwnedWriteHalf, config: Config) -> Option<()> {
    'session: loop {
        let rules = GameRules::default();
        let mut game = Game::new(rules);
        let mut controller = SimulatedController::new(50, 20);
        let displayer = GridDisplayer::new(1234);
        '_game: loop {
            if turn(
                &mut game,
                config.delay,
                &displayer,
                &mut write,
                &mut controller,
            )
            .await
            .is_none()
            {
                continue 'session;
            }
        }
    }
}

async fn turn(
    game: &mut Game,
    delay: Duration,
    displayer: &GridDisplayer,
    write: &mut OwnedWriteHalf,
    controller: &mut SimulatedController,
) -> Option<()> {
    let display = displayer.display(game.get_board());
    let clear = clear::All;
    let goto = cursor::Goto(1, 1);
    let frame = format!("{clear}{goto}{display}");
    write.write_all(frame.as_bytes()).await.ok()?;

    let movement = controller.next_move(&*game).ok()?;
    game.turn(movement).ok()?;
    sleep(delay).await;
    Some(())
}

#[tokio::main]
async fn main() {
    let url = env::var("URL").unwrap_or_else(|_| "localhost:2048".to_string());
    let delay = env::var("DELAY")
        .map(|s| s.parse::<u64>().unwrap())
        .unwrap_or(500);
    let delay = Duration::from_millis(delay);
    let config = Config { delay };
    let server = TcpListener::bind(url).await.unwrap();
    loop {
        let (stream, address) = server.accept().await.unwrap();
        println!("[server] connection from '{address}'");
        let (_, write) = stream.into_split();
        tokio::spawn(game(write, config.clone()));
    }
}
